<div style="font-family: sans-serif; max-width: 600px; margin: 0 auto;">
  <h2>Verify your email address</h2>
  <p>Hi {{ name }},</p>
  <p>Please verify your email address by clicking the link below:</p>
  <p>
    <a href="{{ url }}" style="display: inline-block; padding: 12px 24px;
       background-color: #4F46E5; color: white; text-decoration: none;
       border-radius: 6px;">Verify Email</a>
  </p>
  <p>This link will expire in {{ expires_in_minutes }} minutes.</p>
  <p>If you did not create an account, you can ignore this email.</p>
</div>
